from setuptools import setup

# Quick Start (packaging project)
# python setup.py sdist bdist_wheel
# python -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# python -m twine upload dist/*

setup()
